#!/usr/bin/rexx
--	OO wrapper for the mathematical functions in the rxMath package
--	with methods renamed to their more commonly used names. Precision
--	of the results is the precision used by the invoker of the methods. 

::class math public subclass string

-- The precision of the arguments and results
-- Default is the  NUMERIC DIGITS default
-- Maximum is rxmath's maximum: 16
-- @example num = .math~new(25,16)
-- @example num~precision = 9
--
::attribute precision

-- Init is called from .math~new. 
-- Since .math is a subclass of .string a peculiar thing happens in that the
-- first argument to ~new is not passed to ~init. That's why only the second 
-- argument ( the precision in digits of arguments and results ) is parsed.
-- Following is an example of how to create a new .math instance:
-- @example .math~new(114.2334,16)
-- which results in a invocation of  ~init as above:
-- @param precision=(.context~digits) - where the default is taken from .context
::method init
	use strict arg precision=(.context~digits)
	self~init:super
	if (self+0)~datatype\='NUM' then do	--if self is not a number
		raise syntax 93.904 array(1,self) -- self+0 by itself will		
	end									-- raise a syntax error
	if (precision~datatype('W') & precision>0) then do 
		precision = precision~min(16)	-- adjust maximum to rxmath's maximum
		self~precision = precision
	end
	else do
		self~precision = .context~digits	
	end
	
-- Turns Degrees into Radians
-- @return aNum - angle in Radians
-- @example aRad = .math~new(30)~toRadians 
::method toRadians
	numeric digits self~precision
	return self*self~pi/180
	
-- Turns Degrees into Radians
-- @return aNum - angle in Radians
-- @example radians = .math~new(30)~fromDegrees
::method fromDegrees
	numeric digits self~precision
	return self*self~pi/180
	
-- Turns Radians into Degrees
-- @return aNum - angle in Degrees
-- @example degrees = .math~new(3.5467)~toDegrees
::method toDegrees
	numeric digits self~precision
	return self*180/self~pi	
	
-- Turns Radians into Degrees
-- @return aNum - angle in Degrees
-- @example degrees = .math~new(3.5467)~fromRadians
::method fromRadians
	numeric digits self~precision
	return self*180/self~pi	

-- Yields the value of PI
-- @return Pi 
-- @example PI = .math~pi 
::method pi class
	return RxCalcPi(self~precision)

-- Gives the value of Pi
-- @return Pi
-- @example angle = .math~new(30); PI = angle~pi 
::method pi 
	return RxCalcPi(self~precision)
	
-- Calculates this objects square root
-- @return aNum - being self's square root 
-- @example number = .math~new(16); root = number~sqrt
::method sqrt
	return RxCalcSqrt(self,self~precision)
	
-- Returns the exponential function (e**x)
-- @return aNum - representing e**(self)
-- @example x=.math~new(somenumber); etopowerx = x~exp 
::method exp
	return RxCalcExp(self,self~precision)
	
-- Calculates a number raised to a specified power
-- @param power - the power to which self will be raised
-- @return aNum - self raised to the power specified as argument
-- @example n = .math~new(aNumber); powered = n~power(6.5)
::method power
	use strict arg power
	return RxCalcPower(self,power,self~precision)

-- Retrieves a number's natural logarithm (inverse of exp)
-- @return aNum - self's natural logarithm
-- @example result = .math~new(aNumber)~log
::method log
	return RxCalcLog(self,self~precision)
	
-- Returns the base 10 logarithm
-- @return aNum - self's base10 logarithm
-- @example result = .math~new(4.6765434)~log10
::method log10
	return RxCalcLog10(self,self~precision)
	
-- Determines the sine, where self's angle is in degrees(D), radians(R) or grades(G)
-- @param type='D' - type of self's angle unit (D, R or G), D is default
-- @return aNum - the sine value
-- @example aSin = .math~new(45)~sin('D')
::method sin
	use strict arg type='D'
	return RxCalcSin(self,self~precision,type)

-- Result is the cosine, where self's angle is in degrees(D), radians(R) or grades(G)
-- @param type='D' - type of self's angle unit (D, R or G), D is default
-- @return aNum - the cosine value
-- @example aCos = .math~new(1)~cos('R')
::method cos
	use strict arg type='D'
	return RxCalcCos(self,self~precision,type)

-- Returns the tangent, where self's angle is in degrees(D), radians(R) or grades(G):
-- @param type='D' - type of self's angle unit (D, R or G), D is default
-- @return aNum - the tangent value
-- @example aTan = .math~new(50)~tan('G')
::method tan
	use strict arg type='D'
	return RxCalcTan(self,self~precision,type)

-- Returns the cotangent, where self's angle is in degrees(D), radians(R) or grades(G):
-- @param type='D' - type of self's angle unit (D, R or G), D is default
-- @return aNum - the cotangent value
-- @example aCotan = .math~new(50)~cotan
::method cotan
	use strict arg type='D'
	return RxCalcCotan(self,self~precision,type)

-- Finds the hyperbolic sine
-- @return aNum - self's hyperbolic sine value
-- @example aSinh = .math~new(aNumber)~sinh
::method sinh
	return RxCalcSinH(self,self~precision)

-- Returns the hyperbolic cosine
-- @return aNum - self's hyperbolic cosine value
-- @example aCosh = .math~new(aNumber)~cosh
::method cosh
	return RxCalcCosH(self,self~precision)

-- Calculates the hyperbolic tangent
-- @return aNum - self's hyperbolic tangent
-- @example aTanh = .math~new(aNumber)~tanh
::method tanh
	return RxCalcTanH(self,self~precision)

-- Yields the arcsine, where the result is in degrees(D), radians(R) or grades(G)
-- @param type='D' - result's unit (D, R or G), D is default
-- @return aNum - self's arcsine value in the unit specified by type
-- @example aRad = .math~new(57.2)~asin('R')
::method asin
	use strict arg type='D'
	return RxCalcArcSin(self,self~precision,type)

-- Gives the arccosine, where the result is in degrees(D), radians(R) or grades(G)
-- @param type='D' - result's unit (D, R or G), D is default
-- @return aNum - self's arccosine value in the unit specified by type
-- @example aDeg = .math~new(aNumber)~acos
::method acos
	use strict arg type='D'
	return RxCalcArcCos(self,self~precision,type)

-- Returns the arctangent, where the result is in degrees(D), radians(R) or grades(G)
-- @param type='D' - result's unit (D, R or G), D is default
-- @return aNum - self's arctangent value in the unit specified by type
-- @example aGrade = .math~new(90)~atan('G')
::method atan
	use strict arg type='D'
	return RxCalcArcTan(self,self~precision,type)

::requires	'rxmath' LIBRARY

